/*
 * robot5.c: robot control via command line input
 *
 * Compile with `gcc -O2 robot5.c -o robot5',
 * and run as root with `./robot5'.
 */

#include <stdio.h>
#include <unistd.h>
#include <sys/io.h>

#define BASEPORT 0x378 /* Parallel port for Armdroid 1000 */
#define time2kill 500000

void kill_time(int);	// int is number of times to loop 
void move(int,int,int);     // 1st value is motor number,
                        // 2nd is direction (1 or -1) 
			// 3rd value is steps

void wrist(int,int);    // 1st value is move to make: 1=down, 2=up, 3=ccw 4=cw
			// 2nd value number of times to loop

unsigned char motor[8]={0x00,0x02,0x10,0x12,0x20,0x22,0x30,0x32};
unsigned char step[4]={0x04,0x80,0x08,0x40};
//unsigned char step[4]={0xC8,0x4C,0xC4,0x8C};

#define BASE 0
#define ELBOW 1
#define WRIST 2
#define WRIST_ROTATION 3
#define SHOULDER 4
#define GRIPPER 5

int main(int argc, char *argv[])
{
   char user;
   int move_size;
   int data[6],i,temp;
   FILE *cfPtr;

   //see if robot is in use
   do
   {
      cfPtr = fopen("/root/lock.dat", "r");
      fscanf(cfPtr,"%d",&temp);
      printf("%i \n",temp);
      fclose(cfPtr);
      if (temp !=0)
         sleep(1);
   } while(temp != 0);

   // Lock robot from other users
   if ((cfPtr = fopen("/root/lock.dat", "w")) == NULL)
      printf("File could not be opened\n");
   else
   {
      fprintf(cfPtr,"%d\n",1);
   }

   fclose(cfPtr);

   // retrieve current possition
   if ((cfPtr = fopen("/root/robot_position.dat", "r")) == NULL)
      printf("File could not be opened\n");
   else
   {
      for(i=0;i<6;i++)
      {
         fscanf(cfPtr,"%d",&data[i]);
         printf("%d ",data[i]);
      }
      printf("\n");

   }

   fclose(cfPtr);

  /* Get access to the ports */
  if (ioperm(BASEPORT, 3, 1)) {perror("ioperm"); exit(1);}

  while((user = getopt(argc, argv, "S:F:G:H:N:B:V:X:W:E:R:T:Q")) != -1)
  {

    switch(user)
    {
      case 'S':
      {
        move_size=strtol(optarg, NULL, 0);    
        move(2,1,move_size);
        data[GRIPPER] += move_size;
        break;
      }
      case 'X':
      {
        move_size=strtol(optarg, NULL, 0);    
        move(2,-1,move_size);
        data[GRIPPER] -= move_size;
        break;
      }
      case 'F':
      {
        move_size=strtol(optarg, NULL, 0);    
        move(4,1,move_size);
        data[ELBOW] -= move_size;
        break;
      }
      case 'V':
      {
        move_size=strtol(optarg, NULL, 0);    
        move(4,-1,move_size);
        data[ELBOW] += move_size;
        break;
      }
      case 'G':
      {
        move_size=strtol(optarg, NULL, 0);    
        move(5,1,move_size);
        data[BASE] += move_size;
        break;
      }
      case 'B':
      {
        move_size=strtol(optarg, NULL, 0);    
        move(5,-1,move_size);
        data[BASE] -= move_size;
        break;
      }
      case 'H':
      {
        move_size=strtol(optarg, NULL, 0);    
        move(6,1,move_size);
        data[SHOULDER] += move_size;
        break;
      }
      case 'N':
      {
        move_size=strtol(optarg, NULL, 0);    
        move(6,-1,move_size);
        data[SHOULDER] -= move_size;
        break;
      }
      case 'W':
      {
        move_size=strtol(optarg, NULL, 0);    
        wrist(1,move_size);
        data[WRIST] -= move_size;
        break;
      }
      case 'E':
      {
        move_size=strtol(optarg, NULL, 0);    
        wrist(2,move_size);
        data[WRIST] += move_size;
        break;
      }
      case 'R':
      {
        move_size=strtol(optarg, NULL, 0);    
        wrist(3,move_size);
        data[WRIST_ROTATION] -= move_size;
        break;
      }
      case 'T':
      {
        move_size=strtol(optarg, NULL, 0);    
        wrist(4,move_size);
        data[WRIST_ROTATION] += move_size;
        break;
      }
      case 'Q':
      {
        break;
      }
    }
  }   

  /* We don't need the ports anymore */
  if (ioperm(BASEPORT, 3, 0)) {perror("ioperm"); exit(1);}

   for(i=0;i<6;i++)
   {
      printf("%d ",data[i]);
   }
   printf("\n");


   if ((cfPtr = fopen("/root/robot_position.dat", "w")) == NULL)
      printf("File could not be opened\n");
   else
   {
      for(i=0;i<6;i++)
      {
         fprintf(cfPtr,"%d ",data[i]);
         printf("%d ",data[i]);
         
      }
      fprintf(cfPtr,"\n");
      printf("\n");
   }

   fclose(cfPtr);
   // Unlock robot from other users
   if ((cfPtr = fopen("/root/lock.dat", "w")) == NULL)
      printf("File could not be opened\n");
   else
   {
      fprintf(cfPtr,"%d\n",0);
   }

   fclose(cfPtr);

   return 0;
}

void kill_time (int y)
{
   int x;

   for(x=0;x<y;x++)
   {
   }
}

void move(int m,int direction,int steps)
{
  int x;
  int s = 0;
  unsigned char output;

  for(x=0;x<steps;x++)
  {
     /* Set the data signals (D0-7) of the port*/
     if(direction == 1)
     {
        s++;
        if(s==4)
          s=0;
     }
     else
     {
        s--;
        if(s==-1)
          s=3;
     }

     output=step[s]|motor[m];
     outb(output, BASEPORT);
     kill_time(time2kill);
     output = output | 0x01;
     outb(output, BASEPORT);
     kill_time(time2kill);

  }
  output=0xCC|motor[m];
  outb(output,BASEPORT);    /* Turn off all motors */
  kill_time(time2kill);
  output = output | 0x01;
  outb(output,BASEPORT);
  kill_time(time2kill);
}

void wrist(int wmove,int count)
{
  int x;

  if(wmove==1)
  {
    for(x=0;x<count/10;x++)
    {
       move(1,1,10);
       move(3,-1,10);
    }   
  }
  else if(wmove==2)
  {
    for(x=0;x<count/10;x++)
    {
       move(1,-1,10);
       move(3,1,10);
    }
  }   
  else if(wmove==3)
  {
    for(x=0;x<count/10;x++)
    {
       move(1,1,10);
       move(3,1,10);
    }
  }   
  else if(wmove==4)
  {
    for(x=0;x<count/10;x++)
    {
       move(1,-1,10);
       move(3,-1,10);
    }   
  }
}

