/*
 * robot7.c: robot control via command line input
 *
 * Compile with `gcc -lm robot7.c -o robot7',
 * and run as root with `./robot7'.
 */

#include <stdio.h>
#include <unistd.h>
#include <sys/io.h>
#include <math.h>
#define pi 3.14159265359

typedef struct {
   double r;
   double t;
   double x;
   double y;
} Position;



#define BASEPORT 0x378 /* Parallel port for Armdroid 1000 */
#define time2kill 500000

void kill_time(int);	// int is number of times to loop 
void move(int,int,int);     // 1st value is motor number,
                        // 2nd is direction (1 or -1) 
			// 3rd value is steps

void wrist_move(int,int);    // 1st value is move to make: 1=down, 2=up, 3=ccw 4=cw
			// 2nd value number of times to loop

void lock_shoulder(void); // Lock shoulder joint by turning on stepper motor
void unlock_shoulder(void); // unLock shoulder joint by turning off stepper motor

void rect(double *,double *,double *,double *);
void polar(double *,double *,double *,double *);
void calc(void);

unsigned char motor[8]={0x00,0x02,0x10,0x12,0x20,0x22,0x30,0x32};
unsigned char step[4]={0x04,0x80,0x08,0x40};
//unsigned char step[4]={0xC8,0x4C,0xC4,0x8C};

Position base,elbow,wrist,shoulder,total;
int data[6];

#define BASE 0
#define ELBOW 1
#define WRIST 2
#define WRIST_ROTATION 3
#define SHOULDER 4
#define GRIPPER 5

int main(int argc, char *argv[])
{
   char user;
   int move_size;
   int i,temp,error=0;
   FILE *cfPtr;

   //see if robot is in use
   do
   {
      cfPtr = fopen("/home/robot/public_html/lock.dat", "r");
      fscanf(cfPtr,"%d",&temp);
      fclose(cfPtr);
      if (temp !=0)
         sleep(1);
   } while(temp != 0);

   // Lock robot from other users
   if ((cfPtr = fopen("/home/robot/public_html/lock.dat", "w")) == NULL)
      error=15;
   else
   {
      fprintf(cfPtr,"%d\n",1);
   }

   fclose(cfPtr);

   // retrieve current possition
   if ((cfPtr = fopen("/home/robot/public_html/robot_position.dat", "r")) == NULL)
      error=16;
   else
   {
      for(i=0;i<6;i++)
      {
         fscanf(cfPtr,"%d",&data[i]);
      }

   }

   fclose(cfPtr);

  /* Get access to the ports */
  if (ioperm(BASEPORT, 3, 1)) {perror("ioperm"); exit(1);}

  while((user = getopt(argc, argv, "S:F:G:H:N:B:V:X:W:E:R:T:Q")) != -1)
  {
    lock_shoulder();    

    switch(user)
    {
      case 'S':                   // Gripper Open
      {
        move_size=strtol(optarg, NULL, 0);    
        calc();
        if ((data[GRIPPER]+move_size)<=1600)
        {
           move(2,1,move_size);
           data[GRIPPER] += move_size;
        }
        else
        {
           error=88;
        }
        break;
      }
      case 'X':                   // Gripper Close
      {
        move_size=strtol(optarg, NULL, 0);    
        calc();
        if ((data[GRIPPER]-move_size)>=-2000)
        {
           move(2,-1,move_size);
           data[GRIPPER] -= move_size;
        }
        else
        {
           error=88;
        }
        break;
      }
      case 'F':                   // Elbow Down
      {
        move_size=strtol(optarg, NULL, 0);    
        data[ELBOW] -= move_size;
        calc();
        if((total.y>-10) && (-elbow.t+wrist.t<80))
        {        
           move(4,1,move_size);
        }
        else
        {
           data[ELBOW] += move_size;
           error=88;
        }
        break;
      }
      case 'V':                   // Elbow Up
      {
        move_size=strtol(optarg, NULL, 0);    
        data[ELBOW] += move_size;
        calc();
        if((total.y<10) && (elbow.t-shoulder.t<50))
        {        
           move(4,-1,move_size);
        }
        else
        {
           data[ELBOW] -= move_size;
           error=88;
        }
        break;
      }
      case 'G':                   // Base CW
      {
        move_size=strtol(optarg, NULL, 0);    
        calc();
        if (data[BASE]+move_size <= 800)
        {
           move(5,1,move_size);
           data[BASE] += move_size;
        }
        else
        {
           error=88;
        }
        break;
      }
      case 'B':                   // Base CCW
      {
        move_size=strtol(optarg, NULL, 0);    
        calc();
        if (data[BASE]-move_size >= -800)
        {
           move(5,-1,move_size);
           data[BASE] -= move_size;
        }
        else
        {
           error=88;
        }
        break;
      }
      case 'H':                   // Shoulder Up
      {
        move_size=strtol(optarg, NULL, 0);    
        data[SHOULDER] += move_size;
        calc();
        if(total.y<10)
        {        
           move(6,1,move_size);
        }
        else
        {
           data[SHOULDER] -= move_size;
           error=88;
        }
        break;
      }
      case 'N':                   // Shoulder Down
      {
        move_size=strtol(optarg, NULL, 0);    
        data[SHOULDER] -= move_size;
        calc();
        if((total.y>-10.5) && (shoulder.t>-40) && (elbow.t-shoulder.t<50))
        {        
           move(6,-1,move_size);
        }
        else
        {
           data[SHOULDER] += move_size;
           error=88;
        }
        break;
      }
      case 'W':                   // Wrist Down
      {
        move_size=strtol(optarg, NULL, 0);    
        data[WRIST] -= move_size;
        calc();
        if(total.y>-10.5)
        {        
           wrist_move(1,move_size);
        }
        else
        {
           data[WRIST] += move_size;
           error=88;
        }
        break;
      }
      case 'E':                   // Wrist Up
      {
        move_size=strtol(optarg, NULL, 0);    
        data[WRIST] += move_size;
        calc();
        if(total.y<10)
        {        
           wrist_move(2,move_size);
        }
        else
        {
           data[WRIST] -= move_size;
           error=88;
        }
        break;
      }
      case 'R':                   // Wrist CCW
      {
        move_size=strtol(optarg, NULL, 0);    
        calc();
        if (data[WRIST_ROTATION]-move_size >= -2000)
        { 
           wrist_move(3,move_size);
           data[WRIST_ROTATION] -= move_size;
        }
        else
        {
           error=88;
        }
        break;
      }
      case 'T':                   // Wrist CW
      {
        move_size=strtol(optarg, NULL, 0);    
        calc();
        if (data[WRIST_ROTATION]+move_size <= 2000)
        { 
           wrist_move(4,move_size);
           data[WRIST_ROTATION] += move_size;
        }
        else
        {
           error=88;
        }
        break;
      }
      case 'Q':                   // Quit (do nothing)
      {
        calc();
        break;
      }
    }
  }   

  unlock_shoulder();

  /* We don't need the ports anymore */
  if (ioperm(BASEPORT, 3, 0)) {perror("ioperm"); exit(1);}

   if ((cfPtr = fopen("/home/robot/public_html/robot_position.dat", "w")) == NULL)
      error=17;
   else
   {
      for(i=0;i<6;i++)
      {
         fprintf(cfPtr,"%d ",data[i]);
         if(i<5)
         {
            printf("%d,",data[i]);
         }
         else
         {
            printf("%d\n",data[i]);
         }         
      }
      fprintf(cfPtr,"\n");
   }

   fclose(cfPtr);
   // Unlock robot from other users
   if ((cfPtr = fopen("/home/robot/public_html/lock.dat", "w")) == NULL)
      error=18;
   else
   {
      fprintf(cfPtr,"%d\n",0);
   }

   fclose(cfPtr);

   return error;
}

void kill_time (int y)
{
   int x;

   for(x=0;x<y;x++)
   {
   }
}

void move(int m,int direction,int steps)
{
  int x;
  int s = 0;
  unsigned char output;

  for(x=0;x<steps;x++)
  {
     /* Set the data signals (D0-7) of the port*/
     if(direction == 1)
     {
        s++;
        if(s==4)
          s=0;
     }
     else
     {
        s--;
        if(s==-1)
          s=3;
     }

     output=step[s]|motor[m];
     outb(output, BASEPORT);
     kill_time(time2kill);
     output = output | 0x01;
     outb(output, BASEPORT);
     kill_time(time2kill);

  }
  output=0xCC|motor[m];
  outb(output,BASEPORT);    /* Turn off all motors */
  kill_time(time2kill);
  output = output | 0x01;
  outb(output,BASEPORT);
  kill_time(time2kill);
}

void wrist_move(int wmove,int count)
{
  int x;

  if(wmove==1)
  {
    for(x=0;x<count/10;x++)
    {
       move(1,1,10);
       move(3,-1,10);
    }   
  }
  else if(wmove==2)
  {
    for(x=0;x<count/10;x++)
    {
       move(1,-1,10);
       move(3,1,10);
    }
  }   
  else if(wmove==3)
  {
    for(x=0;x<count/10;x++)
    {
       move(1,1,10);
       move(3,1,10);
    }
  }   
  else if(wmove==4)
  {
    for(x=0;x<count/10;x++)
    {
       move(1,-1,10);
       move(3,-1,10);
    }   
  }
}

void lock_shoulder(void)
{
     int s,m,output;
     s=0;
     m=6;
     output=step[s]|motor[m];
     outb(output, BASEPORT);
     kill_time(time2kill);
     output = output | 0x01;
     outb(output, BASEPORT);
     kill_time(time2kill);
}

void unlock_shoulder(void)
{
     int s,m,output;
     s=0;
     m=6;
     output=0xCC|motor[m];
     outb(output,BASEPORT);    /* Turn off all motors */
     kill_time(time2kill);
     output = output | 0x01;
     outb(output,BASEPORT);
     kill_time(time2kill);
}



void calc(void)
{
   double r, t, x, y;

   base.x=0;
   base.y=9.5;
   base.r=9.5;
   base.t=0;
   printf("%lg,%lg,%lg,%lg\n",base.x,base.y,base.r,base.t);

   t=shoulder.t=(float)data[4]*0.0407;
   r=shoulder.r=7.5;
   rect(&r,&t,&x,&y);      
   shoulder.x=x;
   shoulder.y=y;
   printf("%lg,%lg,%lg,%lg\n",shoulder.x,shoulder.y,shoulder.r,shoulder.t);

   t=elbow.t=(float)data[1]*0.0324;
   r=elbow.r=7.5;
   rect(&r,&t,&x,&y);      
   elbow.x=x;
   elbow.y=y;
   printf("%lg,%lg,%lg,%lg\n",elbow.x,elbow.y,elbow.r,elbow.t);

   t=wrist.t=(float)data[2]*0.0402;
   r=wrist.r=4.5;
   rect(&r,&t,&x,&y);      
   wrist.x=x;
   wrist.y=y;
   printf("%lg,%lg,%lg,%lg\n",wrist.x,wrist.y,wrist.r,wrist.t);

   total.x=wrist.x+elbow.x+shoulder.x;
   total.y=wrist.y+elbow.y+shoulder.y;
   x=total.x;
   y=total.y;
   polar(&x,&y,&r,&t);
   total.r=r;
   total.t=t;
   printf("%lg,%lg,%lg,%lg\n",total.x,total.y,total.r,total.t);
}


/*	Polar/Rectangular conversion
	A program to convert between Polar and Rectangular
	coordinates, with a menu to allow the user to choose
	which conversion will be performed.
	
	Author: Daniel Soskel
	Date: 3/2/09  */
	
/* Function to convert from polar to rect */

void rect(double *r, double *t, double *x, double *y)
{
   *x = *r*(cos(*t*pi/180));
   *y = *r*(sin(*t*pi/180));
}

/* Function to convert from rect to polar */

void polar(double *x, double *y, double *r, double *t)
{
   *r = sqrt((*x * *x)+(*y * *y));
   *t = atan(*y / *x);
   *t = *t * (180/pi);		/* Convert radians to degrees */
}
